// SmileyAnimation.java
//
// ICS 21 / CSE 21 Summer 2010
// Lab Assignment 3
//
// Originally implemented by Norm Jacobson, October 2006
// Minor modifications introduced by Alex Thornton, Summer 2009

import java.awt.Color;
import java.util.ArrayList;


// A SmileyAnimation represents an animation in which a collection of
// smiley faces race one another back and forth a certain number of
// times.

public class SmileyAnimation
{
	// The duration, in milliseconds, that the animation will pause
	// between animation frames.
	private static final int DELAY = 20;

	// The number of laps that each smiley will complete before the
	// race is over.
	public static final long LAPS_TO_RUN = 4;

	// A representation of the window in which the animation appears.
	private SmileyDisplay display;

	// The list of smileys that are competing in the race.
	private ArrayList<SmileyRacer> racers;

	
	// Information for the statistics area, shown after the race is
	// concluded.
	
	// Title of the statistics area.
	private String statisticsTitle;
	
	// The average time, in ticks, it took for a smiley to complete
	// the race.
	private double averageTicks;
	
	// The number of ticks that the fastest smiley took to complete
	// the race.
	private int fewestTicks;
	
	// The name of the fastest smiley.
	private String fastestSmileyName;

	// The number of ticks that the slowest smiley took to complete
	// the race.
	private int mostTicks;
	
	// The name of the slowest smiley.
	private String slowestSmileyName;
	
	
	// This constructor initializes the race, performing a variety of set-up
	// tasks.
	//
	// Your program should work for anywhere from one up to tens of smileys.
	// Be sure to size the smileys so that they all fit on the screen and do
	// not overlap each other.
	//
	// All smiley faces must have the mouth set to the background color of
	// the SmileyDisplay and should initially be constructed so that they're
	// facing to the right (i.e., their mouth is on the right-hand side of
	// the face so it looks like an open mouth, only the right eye is
	// visible).
	//
	// Remember, there are two ways to build a SmileyRacer:
	//
	// (1) Build a SmileyFace first -- including setting its attributes --
	//     then "feed it" to the SmileyRacer constructor.
	// (2) Feed the SmileyRacer constructor an existing SmileyRacer.
	//
	// Remember, also, that SmileyRacers *are* SmileyFaces, so any method
	// you can call on a SmileyFace can also be called on a SmileyRacer.
	public SmileyAnimation(SmileyDisplay d)
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}

	
	// animate() runs the animation.  For each tick until all racers have
	// completed all of their laps, move the racers forward the distance
	// they should go in that tick, based on their current speeds.  The
	// following algorithm will solve this problem:
	//
	//     Until all smileys have finished the race...
	//     Each time through the loop is one 'tick' of the race clock
	//         For each racer in the list of racers...
	//            If the racer has not yet finished the race...
	//                Move the racer forward one clock tick
	//         Repaint the scren to show the movement made this tick
	//         Pause to slow the animation to a visible speed
	//     Race done!  Compute the statistics
	//
	public void animate()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// computeRaceStatistics() computes the statistics for this race.  If
	// there were no racers, the title should indicate that no race took
	// place.  Otherwise, the title should be an introduction to the
	// statistics area, then the various statistics should be computed.
	private void computeRaceStatistics()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// computeFastestSmiley() determines which smiley was the fastest,
	// storing the number of ticks and the name for that smiley into
	// the appropriate fields.
	private void computeFastestRunner()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}

	
	// computeSlowestSmiley() determines which smiley was the slowest,
	// storing the number of ticks and the name for that smiley into
	// the appropriate fields.
	private void computeSlowestRunner()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}

	
	// computeAverageTicks() determines the average time, in ticks, each
	// smiley spent completing the race.
	private void computeAverageTime()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}

	
	// allRacersFinished() returns true if all of the racers have finished
	// the race, false otherwise.
	//
	// The algorithm for solving this kind of problem is:
	//
	//     Assume all have finished
	//     Go through the racers, looking for one that hasn't finished
	//     If you find one that hasn't finished, return false
	//     If you never find one, return true
	//
	private boolean allRacersFinished()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}
	
	
	// pause() pauses the animation for the number of milliseconds given
	// by the DELAY constant declared above.
	private void pause()
	{
		try
		{
			Thread.sleep(DELAY);
		}
		catch (InterruptedException e)
		{
		}
	}

	
	// Accessors -- used by the graphics routine to obtain the information
	// that it needs to display

	// getRacers() returns all the racers (with the information).
	public ArrayList<SmileyRacer> getRacers()
	{
		return racers;
	}


	// getStatisticsTitle() returns the title that should be shown in the
	// statistics area of the window.
	public String getStatisticsTitle()
	{
		return statisticsTitle;
	}

	
	// getAverageTicks() returns the average time, in ticks, that each
	// smiley spent completing the race.
	public double getAverageTicks()
	{
		return averageTicks;
	}

	
	// getFewestTicks() returns the number of ticks spent by the fastest
	// smiley in completing the race.
	public int getFewestTicks()
	{
		return fewestTicks;
	}

	
	// getMostTicks() returns the number of ticks spent by the slowest
	// smiley in completing the race.
	public int getMostTicks()
	{
		return mostTicks;
	}

	
	// getFastestSmileyName() returns the name of the fastest smiley.
	public String getFastestSmileyName()
	{
		return fastestSmileyName;
	}


	// getSlowestSmileyName() returns the name of the slowest smiley.
	public String getSlowestSmileyName()
	{
		return slowestSmileyName;
	}
}
